import { Injectable, signal } from '@angular/core';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { environment } from 'src/environments/environment';
import { firstValueFrom } from 'rxjs';

export interface User {
    username: string;
}

@Injectable({
    providedIn: 'root'
})
export class UsersService {
    private readonly url = environment.apiURL + '/users';
    public $user = signal<User | undefined>(undefined);

    public get token() {
        return localStorage.getItem('token') ?? undefined;
    }
    public set token(token: string | undefined) {
        if (token === undefined) localStorage.removeItem('token');
        else localStorage.setItem('token', token);
    }

    public httpOptions<T extends object>(other: T): T & { headers?: HttpHeaders } {
        if (this.token) {
            (other as any).headers ??= {};
            (other as any).headers.authorization = 'Bearer ' + this.token;
        }

        return other;
    }

    public async updateUser() {
        if (this.token) {
            const user = await firstValueFrom(this.http.get<User>(`${this.url}/self`, {
                headers: { authorization: 'Bearer ' + this.token }
            }));
            this.$user.set(user);
        }
        else this.$user.set(undefined);
    }
    public async get(username: string) {
        const user = await firstValueFrom(this.http.get<User>(`${this.url}?username=${username}`, this.httpOptions({})));
        return user;
    }

    public async logoff() {
        if (!this.token) return;
        this.token = undefined;
        this.$user.set(undefined);
    }
    public async login(username: string, password: string) {
        await this.logoff();
        const token = await firstValueFrom(this.http.post<string>(`${this.url}/login`, { username, password }));
        this.token = token;
        await this.updateUser();
    }
    public async signup(username: string, password: string) {
        await this.logoff();
        await firstValueFrom(this.http.post(`${this.url}/signup`, { username, password }));
        await this.updateUser();
    }

    public constructor(private http: HttpClient) {
        this.updateUser();
    }
}
